@@ -30,7 +30,7 @@ header = '''
 '''
 
 macro_start = '''
-#[macro_export(local_inner_macros)]
+#[macro_export]
 #[doc(hidden)]
 macro_rules! py_class_impl {
     // TT muncher macro. Results are accumulated in $info $slots $impls and $members.
@@ -47,14 +47,14 @@ base_case = '''
             $gc:tt,
             /* data: */ [ $( { $data_offset:expr, $data_name:ident, $data_ty:ty, $init_expr:expr, $init_ty:ty } )* ]
         }
-        $slots:tt { $( $imp:item )* } $members:tt
+        $slots:tt { $( $imp:item )* } $members:tt $props:tt
     } => {
-        py_coerce_item! {
+        $crate::py_coerce_item! {
             $($class_visibility)* struct $class { _unsafe_inner: $crate::PyObject }
         }
 
-        py_impl_to_py_object_for_python_object!($class);
-        py_impl_from_py_object_for_python_object!($class);
+        $crate::py_impl_to_py_object_for_python_object!($class);
+        $crate::py_impl_from_py_object_for_python_object!($class);
 
         impl $crate::PythonObject for $class {
             #[inline]
@@ -78,7 +78,7 @@ base_case = '''
             /// Undefined behavior if the input object does not have the expected type.
             #[inline]
             unsafe fn unchecked_downcast_borrow_from<'a>(obj: &'a $crate::PyObject) -> &'a Self {
-                ::std::mem::transmute(obj)
+                std::mem::transmute(obj)
             }
         }
 
@@ -90,7 +90,7 @@ base_case = '''
                 } else {
                     Err($crate::PythonObjectDowncastError::new(
                         py,
-                        _cpython__py_class__py_class_impl__stringify!($class),
+                        stringify!($class),
                         obj.get_type(py),
                     ))
                 }
@@ -99,18 +99,18 @@ base_case = '''
             #[inline]
             fn downcast_borrow_from<'a, 'p>(py: $crate::Python<'p>, obj: &'a $crate::PyObject) -> Result<&'a $class, $crate::PythonObjectDowncastError<'p>> {
                 if py.get_type::<$class>().is_instance(py, obj) {
-                    unsafe { Ok(::std::mem::transmute(obj)) }
+                    unsafe { Ok(std::mem::transmute(obj)) }
                 } else {
                     Err($crate::PythonObjectDowncastError::new(
                         py,
-                        _cpython__py_class__py_class_impl__stringify!($class),
+                        stringify!($class),
                         obj.get_type(py),
                     ))
                 }
             }
         }
 
-        py_coerce_item! {
+        $crate::py_coerce_item! {
             impl $crate::py_class::BaseObject for $class {
                 type InitType = ( $( $init_ty, )* );
 
@@ -137,7 +137,7 @@ base_case = '''
             }
         }
         $($imp)*
-        py_coerce_item! {
+        $crate::py_coerce_item! {
             impl $class {
                 fn create_instance(py: $crate::Python $( , $data_name : $init_ty )* ) -> $crate::PyResult<$class> {
                     let obj = unsafe {
@@ -149,7 +149,7 @@ base_case = '''
 
                     // hide statics in create_instance to avoid name conflicts
                     static mut TYPE_OBJECT : $crate::_detail::ffi::PyTypeObject
-                        = py_class_type_object_static_init!($class, $gc, $slots);
+                        = $crate::py_class_type_object_static_init!($class, $gc, $slots);
                     static mut INIT_ACTIVE: bool = false;
 
                     // trait implementations that need direct access to TYPE_OBJECT
@@ -161,7 +161,7 @@ base_case = '''
                                 } else {
                                     // automatically initialize the class on-demand
                                     <$class as $crate::py_class::PythonObjectFromPyClassMacro>::initialize(py, None)
-                                        .expect(_cpython__py_class__py_class_impl__concat!("An error occurred while initializing class ", _cpython__py_class__py_class_impl__stringify!($class)))
+                                        .expect(concat!("An error occurred while initializing class ", stringify!($class)))
                                 }
                             }
                         }
@@ -173,9 +173,9 @@ base_case = '''
                                 if $crate::py_class::is_ready(py, &TYPE_OBJECT) {
                                     return Ok($crate::PyType::from_type_ptr(py, &mut TYPE_OBJECT));
                                 }
-                                _cpython__py_class__py_class_impl__assert!(!INIT_ACTIVE,
-                                    _cpython__py_class__py_class_impl__concat!("Reentrancy detected: already initializing class ",
-                                    _cpython__py_class__py_class_impl__stringify!($class)));
+                                assert!(!INIT_ACTIVE,
+                                    concat!("Reentrancy detected: already initializing class ",
+                                    stringify!($class)));
                                 INIT_ACTIVE = true;
                                 let res = init(py, module_name);
                                 INIT_ACTIVE = false;
@@ -185,13 +185,13 @@ base_case = '''
 
                         fn add_to_module(py: $crate::Python, module: &$crate::PyModule) -> $crate::PyResult<()> {
                             let ty = <$class as $crate::py_class::PythonObjectFromPyClassMacro>::initialize(py, module.name(py).ok())?;
-                            module.add(py, _cpython__py_class__py_class_impl__stringify!($class), ty)
+                            module.add(py, stringify!($class), ty)
                         }
                     }
 
                     fn init($py: $crate::Python, module_name: Option<&str>) -> $crate::PyResult<$crate::PyType> {
-                        py_class_type_object_dynamic_init!($class, $py, TYPE_OBJECT, module_name, $slots);
-                        py_class_init_members!($class, $py, TYPE_OBJECT, $members);
+                        $crate::py_class_type_object_dynamic_init!($class, $py, TYPE_OBJECT, module_name, $slots $props);
+                        $crate::py_class_init_members!($class, $py, TYPE_OBJECT, $members);
                         unsafe {
                             if $crate::_detail::ffi::PyType_Ready(&mut TYPE_OBJECT) == 0 {
                                 Ok($crate::PyType::from_type_ptr($py, &mut TYPE_OBJECT))
@@ -250,7 +250,7 @@ slot_groups = (
     ('sdi', 'setdelitem', ['sdi_setitem', 'sdi_delitem'])
 )
 
-def generate_case(pattern, old_info=None, new_info=None, new_impl=None, new_slots=None, new_members=None):
+def generate_case(pattern, old_info=None, new_info=None, new_impl=None, new_slots=None, new_members=None, new_props=None):
     write('{ { %s $($tail:tt)* }\n' % pattern)
     write('$class:ident $py:ident')
     if old_info is not None:
@@ -293,7 +293,12 @@ def generate_case(pattern, old_info=None, new_info=None, new_impl=None, new_slot
         write('\n{ $( $member_name:ident = $member_expr:expr; )* }')
     else:
         write('$members:tt')
-    write('\n} => { py_class_impl! {\n')
+    if new_props:
+        write('\n{ [ $( $prop_doc:tt $prop_getter_name:ident: $prop_type:ty, )* ]')
+        write('\n[ $( $prop_setter_name:ident : $prop_setter_value_type:tt => $prop_setter_setter:ident, )* ] }')
+    else:
+        write('$props:tt')
+    write('\n} => { $crate::py_class_impl! {\n')
     write('{ $($tail)* }\n')
     write('$class $py')
     write(new_info or '$info')
@@ -335,6 +340,18 @@ def generate_case(pattern, old_info=None, new_info=None, new_impl=None, new_slot
         write('}')
     else:
         write('$members')
+    if new_props:
+        getters, setters = new_props
+        write('\n/* props: */ {\n')
+        write('[ $( $prop_doc $prop_getter_name: $prop_type, )*\n')
+        for doc, name, ty in getters:
+            write('{ %s } %s: %s,\n' % (doc, name, ty))
+        write(']\n[ $( $prop_setter_name : $prop_setter_value_type => $prop_setter_setter, )*\n')
+        for name, ty, setter in setters:
+            write('%s : [ %s ] => %s,\n' % (name, ty, setter))
+        write(']\n}\n')
+    else:
+        write('$props')
     write('\n}};\n')
 
 def data_decl():
@@ -415,22 +432,22 @@ def generate_class_method(special_name=None, decoration='',
     def impl(with_params, with_docs):
         if with_docs:
             doc_prefix = '$(#[doc=$doc:expr])*'
-            value_suffix = ', { _cpython__py_class__py_class_impl__concat!($($doc, "\\n"),*) }'
+            value_suffix = ', { concat!($($doc, "\\n"),*) }'
         else:
             doc_prefix = value_suffix = ''
         if with_params:
             param_pattern = ', $($p:tt)+'
-            impl = '''py_argparse_parse_plist_impl!{
+            impl = '''$crate::py_argparse_parse_plist_impl!{
                 py_class_impl_item { $class, $py, %s($cls: &$crate::PyType,) $res_type; { $($body)* } }
                 [] ($($p)+,)
             }''' % name_use
-            value = 'py_argparse_parse_plist_impl!{%s {%s} [] ($($p)+,)}' \
+            value = '$crate::py_argparse_parse_plist_impl!{%s {%s} [] ($($p)+,)}' \
                     % (value_macro, value_args + value_suffix)
         else:
             param_pattern = ''
-            impl = 'py_class_impl_item! { $class, $py,%s($cls: &$crate::PyType,) $res_type; { $($body)* } [] }' \
+            impl = '$crate::py_class_impl_item! { $class, $py,%s($cls: &$crate::PyType,) $res_type; { $($body)* } [] }' \
                 % name_use
-            value = '%s!{%s []}' % (value_macro, value_args + value_suffix)
+            value = '$crate::%s!{%s []}' % (value_macro, value_args + value_suffix)
         pattern = '%s def %s ($cls:ident%s) -> $res_type:ty { $( $body:tt )* }' \
             % (doc_prefix + decoration, name_pattern, param_pattern)
         slots = []
@@ -473,7 +490,7 @@ def traverse_and_clear():
         }
         ''',
         new_impl='''
-            py_coerce_item!{
+            $crate::py_coerce_item!{
                 impl $class {
                     fn __traverse__(&$slf,
                         $py: $crate::Python,
@@ -484,9 +501,9 @@ def traverse_and_clear():
             }
         ''')
     generate_case('def __clear__ (&$slf:ident) $body:block',
-        new_slots=[('tp_clear', 'py_class_tp_clear!($class)')],
+        new_slots=[('tp_clear', '$crate::py_class_tp_clear!($class)')],
         new_impl='''
-            py_coerce_item!{
+            $crate::py_coerce_item!{
                 impl $class {
                     fn __clear__(&$slf, $py: $crate::Python) $body
                 }
@@ -500,22 +517,22 @@ def generate_instance_method(special_name=None, decoration='',
     def impl(with_params, with_docs):
         if with_docs:
             doc_prefix = '$(#[doc=$doc:expr])*'
-            value_suffix = ', { _cpython__py_class__py_class_impl__concat!($($doc, "\\n"),*) }'
+            value_suffix = ', { concat!($($doc, "\\n"),*) }'
         else:
             doc_prefix = value_suffix = ''
         if with_params:
             param_pattern = ', $($p:tt)+'
-            impl = '''py_argparse_parse_plist_impl!{
+            impl = '''$crate::py_argparse_parse_plist_impl!{
                 py_class_impl_item { $class, $py, %s(&$slf,) $res_type; { $($body)* } }
                 [] ($($p)+,)
             }''' % name_use
-            value = 'py_argparse_parse_plist_impl!{%s {%s} [] ($($p)+,)}' \
+            value = '$crate::py_argparse_parse_plist_impl!{%s {%s} [] ($($p)+,)}' \
                     % (value_macro, value_args + value_suffix)
         else:
             param_pattern = ''
-            impl = 'py_class_impl_item! { $class, $py, %s(&$slf,) $res_type; { $($body)* } [] }' \
+            impl = '$crate::py_class_impl_item! { $class, $py, %s(&$slf,) $res_type; { $($body)* } [] }' \
                 % name_use
-            value = '%s!{%s []}' % (value_macro, value_args + value_suffix)
+            value = '$crate::%s!{%s []}' % (value_macro, value_args + value_suffix)
         pattern = '%s def %s (&$slf:ident%s) -> $res_type:ty { $( $body:tt )* }' \
             % (doc_prefix + decoration, name_pattern, param_pattern)
         slots = []
@@ -535,15 +552,15 @@ def static_method():
     generate_case(
         '$(#[doc=$doc:expr])* @staticmethod def $name:ident ($($p:tt)*) -> $res_type:ty { $( $body:tt )* }',
         new_impl='''
-            py_argparse_parse_plist!{
+            $crate::py_argparse_parse_plist!{
                 py_class_impl_item { $class, $py, $name() $res_type; { $($body)* } }
                 ($($p)*)
             }
         ''',
         new_members=[('$name', '''
-            py_argparse_parse_plist!{
+            $crate::py_argparse_parse_plist!{
                 py_class_static_method {$py, $class::$name, {
-                    _cpython__py_class__py_class_impl__concat!($($doc, "\\n"),*)
+                    concat!($($doc, "\\n"),*)
                 } }
                 ($($p)*)
             }
@@ -553,32 +570,26 @@ def static_data():
     generate_case('static $name:ident = $init:expr;',
         new_members=[('$name', '$init')])
 
+def property_method():
+    generate_case('$(#[doc=$doc:expr])* @property def $name:ident(&$slf:ident) -> $res_type:ty { $( $body:tt )* }',
+        new_impl='$crate::py_class_impl_item! { $class, $py, $name(&$slf,) $res_type; { $($body)* } [] }',
+        new_props=([('concat!($($doc, "\\n"),*)', '$name', '$res_type')], [])
+    )
+    generate_case('@$name:ident.setter def $setter_name:ident(&$slf:ident, $value:ident : Option<Option<&$value_type:ty>> ) -> $res_type:ty { $( $body:tt )* }',
+        new_impl='$crate::py_class_impl_item! { $class, $py, $setter_name(&$slf,) $res_type; { $($body)* } [{ $value: Option<Option<&$value_type>> = {} }] }',
+        new_props=([], [('$name', 'Option<&$value_type>', '$setter_name')])
+    )
+    generate_case('@$name:ident.setter def $setter_name:ident(&$slf:ident, $value:ident : Option<&$value_type:ty> ) -> $res_type:ty { $( $body:tt )* }',
+        new_impl='$crate::py_class_impl_item! { $class, $py, $setter_name(&$slf,) $res_type; { $($body)* } [{ $value: Option<&$value_type> = {} }] }',
+        new_props=([], [('$name', '&$value_type', '$setter_name')])
+    )
+    generate_case('@$name:ident.setter def $setter_name:ident(&$slf:ident, $value:ident : Option<$value_type:ty> ) -> $res_type:ty { $( $body:tt )* }',
+        new_impl='$crate::py_class_impl_item! { $class, $py, $setter_name(&$slf,) $res_type; { $($body)* } [{ $value: Option<$value_type> = {} }] }',
+        new_props=([], [('$name', '$value_type', '$setter_name')])
+    )
+
 macro_end = '''
 }
-
-#[doc(hidden)]
-#[macro_export]
-macro_rules! _cpython__py_class__py_class_impl__concat {
-    ($($inner:tt)*) => {
-        concat! { $($inner)* }
-    }
-}
-
-#[doc(hidden)]
-#[macro_export]
-macro_rules! _cpython__py_class__py_class_impl__stringify {
-    ($($inner:tt)*) => {
-        stringify! { $($inner)* }
-    }
-}
-
-#[doc(hidden)]
-#[macro_export]
-macro_rules! _cpython__py_class__py_class_impl__assert {
-    ($($inner:tt)*) => {
-        assert! { $($inner)* }
-    }
-}
 '''
 
 def special_method(decorated_function):
@@ -592,7 +603,7 @@ def special_method(decorated_function):
 def error(special_name, msg):
     print('''
     { { def %s $($tail:tt)* } $( $stuff:tt )* } => {
-        py_error! { "%s" }
+        $crate::py_error! { "%s" }
     };''' % (special_name, msg))
 
 @special_method
@@ -608,8 +619,9 @@ def special_class_method(special_name, *args, **kwargs):
     generate_class_method(special_name=special_name, *args, **kwargs)
 
 class Argument(object):
-    def __init__(self, name):
+    def __init__(self, name, allow_ref=True):
         self.name = name
+        self.allow_ref = allow_ref
 
 @special_method
 def operator(special_name, slot,
@@ -619,6 +631,30 @@ def operator(special_name, slot,
     res_ffi_type='*mut $crate::_detail::ffi::PyObject',
     additional_slots=()
 ):
+    def optref(arg, suffix = ""):
+        if arg.allow_ref:
+            return "Option<&$%s_name%s>" % (arg.name, suffix)
+        else:
+            return "$%s_name%s" % (arg.name, suffix)
+
+    def ref(arg, suffix = ""):
+        if arg.allow_ref:
+            return "&$%s_name%s" % (arg.name, suffix)
+        else:
+            return "$%s_name%s" % (arg.name, suffix)
+
+    def normal(arg, suffix = ""):
+        return "$%s_name%s" % (arg.name, suffix)
+
+    if args:
+        operator_impl(special_name, slot, args, res_type, res_conv, res_ffi_type, additional_slots, optref)
+        operator_impl(special_name, slot, args, res_type, res_conv, res_ffi_type, additional_slots, ref)
+    operator_impl(special_name, slot, args, res_type, res_conv, res_ffi_type, additional_slots, normal)
+    # Generate fall-back matcher that produces an error
+    # when using the wrong method signature
+    error('Invalid signature for operator %s' % special_name)(special_name)
+
+def operator_impl(special_name, slot, args, res_type, res_conv, res_ffi_type, additional_slots, typefunc):
     if res_conv is None:
         if res_type == '()':
             res_conv = '$crate::py_class::slots::UnitCallbackConverter'
@@ -629,37 +665,34 @@ def operator(special_name, slot,
         elif res_type == 'PyObject':
             res_conv = '$crate::_detail::PyObjectCallbackConverter'
         else:
-            res_conv = '$crate::_detail::PythonObjectCallbackConverter::<$crate::%s>(::std::marker::PhantomData)' % res_type
+            res_conv = '$crate::_detail::PythonObjectCallbackConverter::<$crate::%s>(std::marker::PhantomData)' % res_type
     arg_pattern = ''
     param_list = []
     for arg in args:
-        arg_pattern += ', ${0}:ident : ${0}_type:ty'.format(arg.name)
-        param_list.append('{{ ${0} : ${0}_type = {{}} }}'.format(arg.name))
+        arg_pattern += ', ${0}:ident : {1}'.format(arg.name, typefunc(arg, ":ty"))
+        param_list.append('{{ ${0} : {1} = {{}} }}'.format(arg.name, typefunc(arg)))
     if slot == 'sq_contains':
-        new_slots = [(slot, 'py_class_contains_slot!($class::%s, $%s_type)' % (special_name, args[0].name))]
+        new_slots = [(slot, '$crate::py_class_contains_slot!($class::%s, [%s])' % (special_name, typefunc(args[0])))]
     elif slot == 'tp_richcompare':
-        new_slots = [(slot, 'py_class_richcompare_slot!($class::%s, $%s_type, %s, %s)'
-                            % (special_name, args[0].name, res_ffi_type, res_conv))]
+        new_slots = [(slot, '$crate::py_class_richcompare_slot!($class::%s, [%s], %s, %s)'
+                            % (special_name, typefunc(args[0]), res_ffi_type, res_conv))]
     elif len(args) == 0:
-        new_slots = [(slot, 'py_class_unary_slot!($class::%s, %s, %s)'
+        new_slots = [(slot, '$crate::py_class_unary_slot!($class::%s, %s, %s)'
                              % (special_name, res_ffi_type, res_conv))]
     elif len(args) == 1:
-        new_slots = [(slot, 'py_class_binary_slot!($class::%s, $%s_type, %s, %s)'
-                             % (special_name, args[0].name, res_ffi_type, res_conv))]
+        new_slots = [(slot, '$crate::py_class_binary_slot!($class::%s, [%s], %s, %s)'
+                             % (special_name, typefunc(args[0]), res_ffi_type, res_conv))]
     elif len(args) == 2:
-        new_slots = [(slot, 'py_class_ternary_slot!($class::%s, $%s_type, $%s_type, %s, %s)'
-                             % (special_name, args[0].name, args[1].name, res_ffi_type, res_conv))]
+        new_slots = [(slot, '$crate::py_class_ternary_slot!($class::%s, [%s], %s, %s, %s)'
+                             % (special_name, typefunc(args[0]), typefunc(args[1]), res_ffi_type, res_conv))]
     else:
         raise ValueError('Unsupported argument count')
     generate_case(
         pattern='def %s(&$slf:ident%s) -> $res_type:ty { $($body:tt)* }' % (special_name, arg_pattern),
-        new_impl='py_class_impl_item! { $class, $py, %s(&$slf,) $res_type; { $($body)* } [%s] }'
+        new_impl='$crate::py_class_impl_item! { $class, $py, %s(&$slf,) $res_type; { $($body)* } [%s] }'
                  % (special_name, ' '.join(param_list)),
         new_slots=new_slots + list(additional_slots)
     )
-    # Generate fall-back matcher that produces an error
-    # when using the wrong method signature
-    error('Invalid signature for operator %s' % special_name)(special_name)
 
 @special_method
 def call_operator(special_name, slot):
@@ -674,9 +707,9 @@ def binary_numeric_operator(special_name, slot):
     generate_case(
         pattern='def %s($left:ident, $right:ident) -> $res_type:ty { $($body:tt)* }'
             % special_name,
-        new_impl='py_class_impl_item! { $class, $py, %s() $res_type; { $($body)* } ' % special_name
+        new_impl='$crate::py_class_impl_item! { $class, $py, %s() $res_type; { $($body)* } ' % special_name
                 +'[ { $left : &$crate::PyObject = {} } { $right : &$crate::PyObject = {} } ] }',
-        new_slots=[(slot, 'py_class_binary_numeric_slot!($class::%s)' % special_name)]
+        new_slots=[(slot, '$crate::py_class_binary_numeric_slot!($class::%s)' % special_name)]
     )
     error('Invalid signature for binary numeric operator %s' % special_name)(special_name)
 
@@ -712,7 +745,7 @@ special_names = {
     '__cmp__': error('__cmp__ is not supported by py_class! use __richcmp__ instead.'),
     '__richcmp__': operator('tp_richcompare',
         res_type='PyObject',
-        args=[Argument('other'), Argument('op')]),
+        args=[Argument('other'), Argument('op', allow_ref = False)]),
     '__hash__': operator('tp_hash',
         res_conv='$crate::py_class::slots::HashConverter',
         res_ffi_type='$crate::Py_hash_t'),
@@ -754,7 +787,7 @@ special_names = {
                 ]),
     '__missing__': normal_method(),
     '__setitem__': operator('sdi_setitem',
-                args=[Argument('key'), Argument('value')],
+                args=[Argument('key'), Argument('value', allow_ref=False)],
                 res_type='()'),
     '__delitem__': operator('sdi_delitem',
                 args=[Argument('key')],
@@ -869,6 +902,7 @@ def main():
         value_args='$py, $class::$name')
     static_method()
     static_data()
+    property_method()
     print(macro_end)
 
 if __name__ == '__main__':
