@@ -1,7 +1,5 @@
 #![allow(dead_code, unused_variables)]
 
-extern crate cpython;
-
 use cpython::_detail::ffi;
 use cpython::*;
 use std::cell::{Cell, RefCell};
@@ -534,6 +532,35 @@ fn sequence() {
     py_assert!(py, c, "c['abc'] == 'abc'");
 }
 
+py_class!(class SequenceRef |py| {
+    def __getitem__(&self, key: &str) -> PyResult<String> {
+        if key.is_empty() {
+            return Err(PyErr::new::<exc::IndexError, NoArgs>(py, NoArgs));
+        }
+        Ok(format!("Item for {}", key))
+    }
+
+    def __delitem__(&self, key: &str) -> PyResult<()> {
+        if key.is_empty() {
+            return Err(PyErr::new::<exc::IndexError, NoArgs>(py, NoArgs));
+        }
+        Ok(())
+    }
+});
+
+#[test]
+fn sequence_ref() {
+    let gil = Python::acquire_gil();
+    let py = gil.python();
+
+    let c = SequenceRef::create_instance(py).unwrap();
+    py_assert!(py, c, "c['abc'] == 'Item for abc'");
+    py_run!(py, c, "del c['abc']");
+    py_expect_exception!(py, c, "c['']", IndexError);
+    py_expect_exception!(py, c, "c[42]", TypeError);
+    py_expect_exception!(py, c, "del c['']", IndexError);
+}
+
 py_class!(class Callable |py| {
     def __call__(&self, arg: i32) -> PyResult<i32> {
         Ok(arg * 6)
@@ -576,6 +603,56 @@ fn setitem() {
     py_expect_exception!(py, c, "del c[1]", NotImplementedError);
 }
 
+py_class!(class SetItemRef |py| {
+    data key: RefCell<String>;
+    data val: Cell<i32>;
+
+    def __setitem__(&self, key: &str, val: i32) -> PyResult<()> {
+        *self.key(py).borrow_mut() = key.to_string();
+        self.val(py).set(val);
+        Ok(())
+    }
+});
+
+#[test]
+fn setitem_ref() {
+    let gil = Python::acquire_gil();
+    let py = gil.python();
+
+    let c = SetItemRef::create_instance(py, RefCell::new(String::new()), Cell::new(0)).unwrap();
+    py_run!(py, c, "c['foo'] = 100");
+    assert_eq!(*c.key(py).borrow(), "foo");
+    assert_eq!(c.val(py).get(), 100);
+    py_expect_exception!(py, c, "c[None] = 1", TypeError);
+}
+
+py_class!(class SetItemOptRef |py| {
+    data key: RefCell<String>;
+    data val: Cell<i32>;
+
+    def __setitem__(&self, key: Option<&str>, val: i32) -> PyResult<()> {
+        if let Some(key) = key {
+            *self.key(py).borrow_mut() = key.to_string();
+            self.val(py).set(val);
+            Ok(())
+        } else {
+            Err(PyErr::new::<exc::IndexError, NoArgs>(py, NoArgs))
+        }
+    }
+});
+
+#[test]
+fn setitem_opt_ref() {
+    let gil = Python::acquire_gil();
+    let py = gil.python();
+
+    let c = SetItemOptRef::create_instance(py, RefCell::new(String::new()), Cell::new(0)).unwrap();
+    py_run!(py, c, "c['foo'] = 100");
+    assert_eq!(*c.key(py).borrow(), "foo");
+    assert_eq!(c.val(py).get(), 100);
+    py_expect_exception!(py, c, "c[None] = 1", IndexError);
+}
+
 py_class!(class DelItem |py| {
     data key: Cell<i32>;
 
@@ -654,6 +731,41 @@ fn contains() {
     py_run!(py, c, "assert 'wrong type' not in c");
 }
 
+py_class!(class ContainsRef |py| {
+    def __contains__(&self, item: &str) -> PyResult<bool> {
+        Ok(item.is_empty())
+    }
+});
+
+#[test]
+fn contains_ref() {
+    let gil = Python::acquire_gil();
+    let py = gil.python();
+
+    let c = ContainsRef::create_instance(py).unwrap();
+    py_run!(py, c, "assert '' in c");
+    py_run!(py, c, "assert 'hello' not in c");
+    py_run!(py, c, "assert 42 not in c");
+}
+
+py_class!(class ContainsOptRef |py| {
+    def __contains__(&self, item: Option<&str>) -> PyResult<bool> {
+        Ok(item.map_or(false, str::is_empty))
+    }
+});
+
+#[test]
+fn contains_opt_ref() {
+    let gil = Python::acquire_gil();
+    let py = gil.python();
+
+    let c = ContainsOptRef::create_instance(py).unwrap();
+    py_run!(py, c, "assert '' in c");
+    py_run!(py, c, "assert 'hello' not in c");
+    py_run!(py, c, "assert 42 not in c");
+    py_run!(py, c, "assert None not in c");
+}
+
 py_class!(class UnaryArithmetic |py| {
     def __neg__(&self) -> PyResult<&'static str> {
         Ok("neg")
@@ -832,6 +944,74 @@ fn rich_comparisons_python_3_type_error() {
     py_expect_exception!(py, c2, "1 >= c2", TypeError);
 }
 
+py_class!(class RichComparisonsRef |py| {
+    def __repr__(&self) -> PyResult<&'static str> {
+        Ok("RCR")
+    }
+
+    def __richcmp__(&self, other: &str, op: CompareOp) -> PyResult<bool> {
+        match op {
+            CompareOp::Lt => Ok("RCR" < other),
+            CompareOp::Le => Ok("RCR" <= other),
+            CompareOp::Eq => Ok("RCR" == other),
+            CompareOp::Ne => Ok("RCR" != other),
+            CompareOp::Gt => Ok("RCR" > other),
+            CompareOp::Ge => Ok("RCR" >= other),
+        }
+    }
+});
+
+#[test]
+fn rich_comparisons_ref() {
+    let gil = Python::acquire_gil();
+    let py = gil.python();
+
+    let c = RichComparisonsRef::create_instance(py).unwrap();
+    py_assert!(py, c, "c < 'ZZZ'");
+    py_assert!(py, c, "c <= 'ZZZ'");
+    py_assert!(py, c, "c <= 'RCR'");
+    py_assert!(py, c, "c == 'RCR'");
+    py_assert!(py, c, "c != 'RRR'");
+    py_assert!(py, c, "c > 'AAA'");
+    py_assert!(py, c, "c >= 'AAA'");
+    py_assert!(py, c, "c >= 'RCR'");
+}
+
+py_class!(class RichComparisonsOptRef |py| {
+    def __repr__(&self) -> PyResult<&'static str> {
+        Ok("RCR")
+    }
+
+    def __richcmp__(&self, other: Option<&str>, op: CompareOp) -> PyResult<bool> {
+        match other {
+            Some(other) => match op {
+                CompareOp::Lt => Ok("RCR" < other),
+                CompareOp::Le => Ok("RCR" <= other),
+                CompareOp::Eq => Ok("RCR" == other),
+                CompareOp::Ne => Ok("RCR" != other),
+                CompareOp::Gt => Ok("RCR" > other),
+                CompareOp::Ge => Ok("RCR" >= other),
+            },
+            None => match op {
+                CompareOp::Ne | CompareOp::Gt | CompareOp::Ge => Ok(true),
+                CompareOp::Eq | CompareOp::Lt | CompareOp::Le => Ok(false),
+            },
+        }
+    }
+});
+
+#[test]
+fn rich_comparisons_opt_ref() {
+    let gil = Python::acquire_gil();
+    let py = gil.python();
+
+    let c = RichComparisonsOptRef::create_instance(py).unwrap();
+    py_assert!(py, c, "c != None");
+    py_assert!(py, c, "c == 'RCR'");
+    py_assert!(py, c, "c > None");
+    py_assert!(py, c, "None <= c");
+}
+
 py_class!(class InPlaceOperations |py| {
     data value: Cell<u32>;
 
@@ -942,6 +1122,33 @@ fn inplace_operations() {
     );
 }
 
+py_class!(class InPlaceOperationsRef |py| {
+    data value: RefCell<String>;
+
+    def __repr__(&self) -> PyResult<String> {
+        Ok(format!("IPOR({:?})", self.value(py).borrow()))
+    }
+
+    def __iadd__(&self, other: &str) -> PyResult<Self> {
+        let mut value = self.value(py).borrow_mut();
+        value.push_str(other);
+        Ok(self.clone_ref(py))
+    }
+});
+
+#[test]
+fn inplace_operations_ref() {
+    let gil = Python::acquire_gil();
+    let py = gil.python();
+
+    let c = InPlaceOperationsRef::create_instance(py, RefCell::new(String::new())).unwrap();
+    py_run!(
+        py,
+        c,
+        r#"d = c; c += "hello"; c += ", world"; assert repr(c) == repr(d) == 'IPOR("hello, world")'"#
+    );
+}
+
 py_class!(class ContextManager |py| {
     data exit_called : Cell<bool>;
 
@@ -981,3 +1188,91 @@ fn context_manager() {
     );
     assert!(c.exit_called(py).get());
 }
+
+py_class!(class Properties |py| {
+    data value: Cell<i32>;
+    data value_by_ref: RefCell<String>;
+    data value_by_opt_ref: RefCell<String>;
+
+    def __repr__(&self) -> PyResult<String> {
+        Ok(format!("P({:?} {:?} {:?})",
+            self.value(py).get(),
+            self.value_by_ref(py).borrow(),
+            self.value_by_opt_ref(py).borrow()))
+    }
+
+    @property def prop(&self) -> PyResult<i32> {
+        Ok(self.value(py).get())
+    }
+
+    @prop.setter def set_prop(&self, value: Option<i32>) -> PyResult<()> {
+        self.value(py).set(value.unwrap_or(0));
+        Ok(())
+    }
+
+    @property def prop_by_ref(&self) -> PyResult<String> {
+        Ok(self.value_by_ref(py).borrow().to_string())
+    }
+
+    @prop_by_ref.setter def set_prop_by_ref(&self, value: Option<&str>) -> PyResult<()> {
+        *self.value_by_ref(py).borrow_mut() = value.unwrap_or("DELETED").to_string();
+        Ok(())
+    }
+
+    @property def prop_by_opt_ref(&self) -> PyResult<String> {
+        Ok(self.value_by_opt_ref(py).borrow().to_string())
+    }
+
+    @prop_by_opt_ref.setter def set_prop_by_opt_ref(&self, value: Option<Option<&str>>) -> PyResult<()> {
+        let value = value.unwrap_or(Some("DELETED")).unwrap_or("NO VALUE");
+        *self.value_by_opt_ref(py).borrow_mut() = value.to_string();
+        Ok(())
+    }
+
+    /// docs for match
+    @property def r#match(&self) -> PyResult<bool> {
+        Ok(self.value(py).get() != 0)
+    }
+});
+
+#[test]
+fn properties() {
+    let gil = Python::acquire_gil();
+    let py = gil.python();
+
+    let c = Properties::create_instance(
+        py,
+        Cell::new(0),
+        RefCell::new(String::new()),
+        RefCell::new(String::new()),
+    )
+    .unwrap();
+
+    py_run!(
+        py,
+        c,
+        "assert 'docs for match' in c.__class__.match.__doc__"
+    );
+
+    py_run!(py, c, "assert c.prop == 0");
+    py_run!(py, c, "assert not c.match");
+    py_run!(py, c, "c.prop = 42");
+    assert_eq!(c.value(py).get(), 42);
+    py_run!(py, c, "assert c.match");
+    assert!(c.r#match(py).unwrap());
+
+    // Instead of really deleting, our setter sets back to 0
+    py_run!(py, c, "delattr(c, 'prop')");
+    py_run!(py, c, "assert c.prop == 0");
+
+    py_run!(py, c, "c.prop_by_ref = 'testing'");
+    py_run!(py, c, "assert c.prop_by_ref == 'testing'");
+
+    py_run!(py, c, "c.prop_by_opt_ref = 'something'");
+    assert_eq!(*c.value_by_opt_ref(py).borrow(), "something");
+    py_run!(py, c, "c.prop_by_opt_ref = None");
+    py_run!(py, c, "repr(c) == 'P(42, \"testing\" \"NO VALUE\")'");
+
+    py_run!(py, c, "del c.prop_by_opt_ref");
+    py_run!(py, c, "repr(c) == 'P(42, \"testing\" \"DELETED\")'");
+}
